Package: ATACseqQC
Type: Package
Title: ATAC-seq Quality Control
Version: 1.37.0
Author: Jianhong Ou, Haibo Liu, Feng Yan, Jun Yu, Michelle Kelliher,
        Lucio Castilla, Nathan Lawson, Lihua Julie Zhu
Maintainer: Jianhong Ou <jou@morgridge.org>
Description: ATAC-seq, an assay for Transposase-Accessible Chromatin
        using sequencing, is a rapid and sensitive method for chromatin
        accessibility analysis. It was developed as an alternative
        method to MNase-seq, FAIRE-seq and DNAse-seq. Comparing to the
        other methods, ATAC-seq requires less amount of the biological
        samples and time to process. In the process of analyzing
        several ATAC-seq dataset produced in our labs, we learned some
        of the unique aspects of the quality assessment for ATAC-seq
        data.To help users to quickly assess whether their ATAC-seq
        experiment is successful, we developed ATACseqQC package
        partially following the guideline published in Nature Method
        2013 (Greenleaf et al.), including diagnostic plot of fragment
        size distribution, proportion of mitochondria reads, nucleosome
        positioning pattern, and CTCF or other Transcript Factor
        footprints.
Depends: R (>= 3.5.0), BiocGenerics, S4Vectors
Imports: BSgenome, Biostrings, ChIPpeakAnno, IRanges, GenomicRanges,
        GenomicAlignments, GenomeInfoDb, GenomicScores, graphics, grid,
        limma, Rsamtools (>= 1.31.2), randomForest, rtracklayer, stats,
        motifStack, preseqR, utils, KernSmooth, edgeR, BiocParallel
Suggests: BiocStyle, knitr, BSgenome.Hsapiens.UCSC.hg19,
        TxDb.Hsapiens.UCSC.hg19.knownGene, phastCons100way.UCSC.hg19,
        MotifDb, trackViewer, testthat, rmarkdown
License: GPL (>= 2)
VignetteBuilder: knitr
RoxygenNote: 7.3.2
biocViews: Sequencing, DNASeq, ATACSeq, GeneRegulation, QualityControl,
        Coverage, NucleosomePositioning, ImmunoOncology
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/ATACseqQC
git_branch: devel
git_last_commit: a1d815a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-29 20:24:42 UTC; biocbuild
